//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "TGameForm.h"
#include "TGameSystem.h"
#include "TGameScreen.h"
#include "TGameSound.h"
#include "TGameTimer.h"
#include "TValueDialog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TGameForm* GameForm = NULL;
//---------------------------------------------------------------------------
static const AnsiString INFO_TITLE = "Forkishable";
static const AnsiString INFO_ABOUT =
	INFO_TITLE + " for Win32 Version 1.0.1\r\n\r\n"
	"Copyright (C) 2003 Moochos. All Rights Reserved.";
static const AnsiString INFO_TIPS =
	"[Rules]\r\n"
	"One point for clicking a dish.\r\n"
	"Another fork comes up for clicking the table.\r\n"
	"Game over for clicking a fork.\r\n"
	"Game over when mouse pointer is out of the game screen.\r\n"
	"\r\n"
	"[Tips]\r\n"
	"To start the game, press Space key or Enter key.";
static const AnsiString FILE_HELP = ExtractFilePath(Application->ExeName) + "Help\\index.html";
static const AnsiString FILE_INI = ExtractFilePath(Application->ExeName) + "Settings.ini";
static const AnsiString INI_SECTION = "Settings";
static const AnsiString INI_KEY = "Values";
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGameForm
//---------------------------------------------------------------------------
// published
//---------------------------------------------------------------------------
void __fastcall TGameForm::ApplicationDeactivate(TObject *Sender)
{
	DiscardGame();
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::FormPaint(TObject *Sender)
{
	GameScreen->Update();
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::FormMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button != mbLeft || FSenseMouseUp) return;

	GameSystem->MouseDown(X, Y);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::FormMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button != mbLeft || !FSenseMouseUp) return;

	GameSystem->MouseDown(X, Y);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	GameSystem->KeyDown(Key);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::MenuItemClick(TObject *Sender)
{
	DiscardGame();

	if(Sender == New_MenuItem || Sender == NewP_MenuItem) {
		ProcNew();
	}
	else if(Sender == Status_MenuItem || Sender == StatusP_MenuItem) {
		ProcShowInfo(SHOWINFO_STATUS);
	}
	else if(Sender == Clear_MenuItem) {
		ProcClear();
	}
	else if(Sender == ForkCount_MenuItem) {
		ProcOptions(OPTIONS_FORKCOUNT);
	}
	else if(Sender == DishCount_MenuItem) {
		ProcOptions(OPTIONS_DISHECOUNT);
	}
	else if(Sender == Speed_MenuItem) {
		ProcOptions(OPTIONS_SPEED);
	}
	else if(Sender == SenseMouseUp_MenuItem) {
		ProcOptions(OPTIONS_SENSEMOUSEUP);
	}
	else if(Sender == RestorePosition_MenuItem) {
		ProcOptions(OPTIONS_RESTOREPOSITION);
	}
	else if(Sender == PlaySound_MenuItem) {
		ProcOptions(OPTIONS_PLAYSOUND);
	}
	else if(Sender == Default_MenuItem) {
		ProcOptions(OPTIONS_DEFAULT);
	}
	else if(Sender == Exit_MenuItem) {
		Close();
	}
	else if(Sender == Help2_MenuItem) {
		ProcShowInfo(SHOWINFO_HELP);
	}
	else if(Sender == Tips_MenuItem) {
		ProcShowInfo(SHOWINFO_TIPS);
	}
	else if(Sender == About_MenuItem) {
		ProcShowInfo(SHOWINFO_ABOUT);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::PopupMenu1Popup(TObject *Sender)
{
	DiscardGame();
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::GameSystemModeChange(TObject *Sender)
{
	PopupMenu = GameSystem->Mode == GAMEMODE_PLAYING ? NULL : PopupMenu1;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::GameTimerTimer(TObject *Sender)
{
	static DWORD timing = timeGetTime() - FSpeed;

	// return if before the timing
	DWORD currentTime = timeGetTime();
	if(currentTime < timing + FSpeed) return;
	timing = currentTime - (FSpeed == 0 ? 0 : (timing % FSpeed));

	// game over if mouse pointer is out of game screen
	if(GameSystem->Mode == GAMEMODE_PLAYING) {
		TPoint point = ScreenToClient(Mouse->CursorPos);
		if(!GameScreen->Contains(point.x, point.y))
			GameSystem->Mode = GAMEMODE_GAMEOVER;
	}

	// move units
	GameSystem->Move();
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TGameForm::SetSpeed(int ASpeed)
{
	FSpeed = ASpeed;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::SetSenseMouseUp(bool ASenseMouseUp)
{
	FSenseMouseUp = ASenseMouseUp;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::SetRestorePosition(bool ARestorePosition)
{
	FRestorePosition = ARestorePosition;
}
//---------------------------------------------------------------------------
int __fastcall TGameForm::Saturate(int AValue, int AMin, int AMax)
{
	if(AValue < AMin) return AMin;
	if(AValue > AMax) return AMax;
	return AValue;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::UpdateState()
{
	SenseMouseUp_MenuItem->Checked = FSenseMouseUp;
	RestorePosition_MenuItem->Checked = FRestorePosition;
	PlaySound_MenuItem->Checked = GameSound->Enabled;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::DiscardGame()
{
	if(GameSystem->Mode == GAMEMODE_PLAYING)
		GameSystem->Mode = GAMEMODE_GAMEOVER;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::ProcNew()
{
	GameSystem->Mode = GAMEMODE_READY;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::ProcClear()
{
	if(Application->MessageBox("Clearing high-score.\r\n\r\nSure?",
		"Clear", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) return;

	GameSystem->HighScore = 0;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::ProcOptions(OPTIONS AType)
{
	AnsiString value;
	TGameOptions options = GameSystem->Options;

	switch(AType) {
		case OPTIONS_FORKCOUNT:
			value = options.ForkCount;
			if(!ValueDialog->Execute("Fork Count", "Enter fork count. (0-32)", value)) return;
			options.ForkCount = Saturate(value.ToIntDef(options.ForkCount), 0, 32);
			GameSystem->Options = options;
			break;

		case OPTIONS_DISHECOUNT:
			value = options.DishCount;
			if(!ValueDialog->Execute("Dish Count", "Enter dish count. (1-32)", value)) return;
			options.DishCount = Saturate(value.ToIntDef(options.DishCount), 1, 32);
			GameSystem->Options = options;
			break;

		case OPTIONS_SPEED:
			value = Speed;
			if(!ValueDialog->Execute("Speed", "Enter game speed. (10-1000 ms)", value)) return;
			Speed = Saturate(value.ToIntDef(Speed), 10, 1000);
			break;

		case OPTIONS_SENSEMOUSEUP:
			SenseMouseUp = !SenseMouseUp;
			break;

		case OPTIONS_RESTOREPOSITION:
			RestorePosition = !RestorePosition;
			break;

		case OPTIONS_PLAYSOUND:
			GameSound->Enabled = !GameSound->Enabled;
			break;

		case OPTIONS_DEFAULT:
			if(Application->MessageBox("Applying default settings.\r\n\r\n"
				"High-score will be cleared.\r\n\r\nSure?", "Default",
				MB_OKCANCEL | MB_ICONQUESTION) != IDOK) return;
			InitSettings();
			break;
	}

	UpdateState();
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::ProcShowInfo(SHOWINFO AType)
{
	switch(AType) {
		case SHOWINFO_STATUS:
			Application->MessageBox((AnsiString("High-Score: ") +
				GameSystem->HighScore).c_str(), "Status", MB_ICONINFORMATION);
			break;

		case SHOWINFO_HELP:
			// confirm open
			if(Application->MessageBox("Opening HTML help file.\r\n\r\nSure?",
				"Help", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) return;

			// open help file
			if(reinterpret_cast<int>(ShellExecute(Application->Handle, NULL,
				FILE_HELP.c_str(), NULL, NULL, SW_SHOWNORMAL)) > 32) return;

			// show error message
			Application->MessageBox(("Failed to open help file: "
				+ FILE_HELP).c_str(), Application->Title.c_str(), MB_ICONEXCLAMATION);
			break;

		case SHOWINFO_TIPS:
			Application->MessageBox(INFO_TIPS.c_str(), "Tips", MB_ICONINFORMATION);
			break;

		case SHOWINFO_ABOUT:
			Application->MessageBox(INFO_ABOUT.c_str(), "About", MB_ICONINFORMATION);
			break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::InitSettings()
{
	Speed = 100;
	SenseMouseUp = false;
	RestorePosition = true;
	GameSystem->HighScore = 0;
	GameSystem->Options = TGameOptions();
	GameSound->Enabled = true;
	UpdateState();
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::LoadSettings()
{
	TMemIniFile* ini = NULL;
	TStringList* values = new TStringList();
	int index = 0;

	__try {
		try {
			// read values
			ini = new TMemIniFile(FILE_INI);
			values->CommaText = ini->ReadString(INI_SECTION, INI_KEY, "");

			// apply values
			Speed = values->Strings[index++].ToInt();
			SenseMouseUp = StrToBool(values->Strings[index++]);
			RestorePosition = StrToBool(values->Strings[index++]);
			GameSystem->HighScore = values->Strings[index++].ToInt();
			TGameOptions options;
			options.ForkCount = values->Strings[index++].ToInt();
			options.DishCount = values->Strings[index++].ToInt();
			GameSystem->Options = options;
			GameSound->Enabled = StrToBool(values->Strings[index++]);
			if(RestorePosition) {
				Left = values->Strings[index++].ToInt();
				Top = values->Strings[index++].ToInt();
			}
			else {
				index += 2;
			}
			UpdateState();
		}
		catch(...) {
			InitSettings();
		}
	}
	__finally {
		delete values;
		delete ini;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::SaveSettings()
{
	TMemIniFile* ini = NULL;
	TStringList* values = new TStringList();

	__try {
		try {
			// prepare values
			values->Add(Speed);
			values->Add(BoolToStr(SenseMouseUp));
			values->Add(BoolToStr(RestorePosition));
			values->Add(GameSystem->HighScore);
			values->Add(GameSystem->Options.ForkCount);
			values->Add(GameSystem->Options.DishCount);
			values->Add(BoolToStr(GameSound->Enabled));
			values->Add(Left);
			values->Add(Top);
			values->Add("");

			// write values
			ini = new TMemIniFile(FILE_INI);
			ini->WriteString(INI_SECTION, INI_KEY, values->CommaText);
			ini->UpdateFile();
		}
		catch(...) {}
	}
	__finally {
		delete values;
		delete ini;
	}
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TGameForm::WMMenuSelect(TMessage& AMessage)
{
	DiscardGame();
}
//---------------------------------------------------------------------------
// public
//---------------------------------------------------------------------------
__fastcall TGameForm::TGameForm(TComponent* Owner) : TForm(Owner), FSpeed(100),
	FSenseMouseUp(false)
{
	// init
	randomize();
	GameSystem = new TGameSystem();
	GameScreen = new TGameScreen();
	GameSound = new TGameSound();
	GameTimer = new TGameTimer(true, 10, &GameTimerTimer);

	Application->Title = INFO_TITLE;
	Application->OnDeactivate = ApplicationDeactivate;
	Caption = INFO_TITLE;
	ClientWidth = SCRR.Width();
	ClientHeight = SCRR.Height();
	GameSystem->OnModeChange = GameSystemModeChange;
	GameScreen->DstCanvas = Canvas;
	GameScreen->UnitImages = ImageList1;
	GameTimer->FreeOnTerminate = false;

	// load settings
	LoadSettings();

	// start game system with title scene
	GameSystem->Mode = GAMEMODE_TITLE;
	GameTimer->Resume();
}
//---------------------------------------------------------------------------
__fastcall TGameForm::~TGameForm()
{
	// quit game system
	GameSystem->Mode = GAMEMODE_QUIT;

	// save settings
	SaveSettings();

	// uninit
	try {
		GameTimer->Terminate();
		GameTimer->WaitFor();
		delete GameTimer; GameTimer = NULL;
	}
	catch(...) {}
	delete GameSound; GameSound = NULL;
	delete GameScreen; GameScreen = NULL;
	delete GameSystem; GameSystem = NULL;
}
//---------------------------------------------------------------------------

