//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TGameSystem.h"
#include "TGameUnit.h"
#include "TGameScreen.h"
#include "TGameSound.h"
//---------------------------------------------------------------------------
TGameSystem* GameSystem = NULL;
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// struct TGameOptions
//---------------------------------------------------------------------------
__fastcall TGameOptions::TGameOptions() : ForkCount(6), DishCount(8)
{
	// do nothing
}
//---------------------------------------------------------------------------
__fastcall TGameOptions::~TGameOptions()
{
	// do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TGameOptions::operator==(const TGameOptions& ASrc) const
{
	return ForkCount == ASrc.ForkCount && DishCount == ASrc.DishCount;
}
//---------------------------------------------------------------------------
bool __fastcall TGameOptions::operator!=(const TGameOptions& ASrc) const
{
	return !(*this == ASrc);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGameSystem
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TGameSystem::SetMode(GAMEMODE AMode)
{
	// uninit current game mode
	if(FMode == GAMEMODE_PLAYING) {
		// show cursor
		Screen->Cursor = crDefault;
	}

	// clear game units
	FUnitList->Clear();

	// set game mode
	FMode = AMode;

	// init new game mode
	if(FMode == GAMEMODE_TITLE) {
		// do nothing
	}
	else if(FMode == GAMEMODE_READY) {
		FScore = 0;
	}
	else if(FMode == GAMEMODE_PLAYING) {
		// hide cursor
		Screen->Cursor = crNone;

		// put game units
		new TTableUnit();
		for(int i = 0; i < FOptions.ForkCount; i++)
			new TForkUnit();
		for(int i = 0; i < FOptions.DishCount; i++)
			new TDishUnit();
	}
	else if(FMode == GAMEMODE_GAMEOVER) {
		if(FScore > FHighScore) FHighScore = FScore;
		GameSound->Play(GAMESOUND_GAMEOVER);
	}

	// show message
	new TMessageUnit();

	// notify mode is changed
	DoModeChange();
}
//---------------------------------------------------------------------------
void __fastcall TGameSystem::DoModeChange()
{
	if(FOnModeChange == NULL) return;

	FOnModeChange(Application);
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameSystem::TGameSystem() : FUnitList(NULL), FScore(0),
	FHighScore(0), FOnModeChange(NULL)
{
	FUnitList = new TGameUnitList();
}
//---------------------------------------------------------------------------
__fastcall TGameSystem::~TGameSystem()
{
	if(Mode != GAMEMODE_QUIT)
		Mode = GAMEMODE_QUIT;

	// uninit
	delete FUnitList; FUnitList = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TGameSystem::Move()
{
	// prepare screen
	GameScreen->Modified = false;

	// move game units
	for(int i = 0; i < FUnitList->Count; i++) {
		if(!FUnitList->Units[i]->Alive) {
			delete FUnitList->Units[i--];
			continue;
		}
		FUnitList->Units[i]->Move();
	}

	// update screen
	if(GameScreen->Modified)
		GameScreen->Update();
}
//---------------------------------------------------------------------------
void __fastcall TGameSystem::MouseDown(int AX, int AY)
{
	if(!GameScreen->Contains(AX, AY)) return;

	if(Mode == GAMEMODE_TITLE) {
		Mode = GAMEMODE_READY;
	}
	else if(Mode == GAMEMODE_READY) {
		Mode = GAMEMODE_PLAYING;
	}
	else if(Mode == GAMEMODE_PLAYING) {
		bool gameOver = false;
		bool score = 0;

		// check units on clicked point
		for(int index = FUnitList->Count - 1; index >= 0; index--) {
			if(!FUnitList->Units[index]->IsHit(AX, AY)) continue;
			switch(FUnitList->Units[index]->Type) {
				case GAMEUNIT_FORK:
					gameOver = true;
					index = 0;
					break;

				case GAMEUNIT_DISH:
					score++;
					FUnitList->Units[index]->Alive = false;
					new TDishUnit();
					break;
			}
		}

		// add score
		if(score > 0) {
			Score += score;
			GameSound->Play(GAMESOUND_SCORE);
		}

		// check game over
		if(gameOver) {
			Mode = GAMEMODE_GAMEOVER;
			return;
		}

		// pay penalty if got no score
		if(score <= 0) {
			new TMarkUnit(AX, AY);
			new TForkUnit();
			GameSound->Play(GAMESOUND_MISS);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameSystem::KeyDown(WORD AKey)
{
	switch(FMode) {
		case GAMEMODE_TITLE:
		case GAMEMODE_GAMEOVER:
			if(AKey == VK_SPACE || AKey == VK_RETURN) 
				GameSystem->Mode = GAMEMODE_READY;
			break;
	}
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
#pragma package(smart_init)
