//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TGameUnit.h"
#include "TGameSystem.h"
#include "TGameScreen.h"
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
inline bool __fastcall TGameUnit::Contains(const TRect& ARect, int AX, int AY) const
{
	return AX >= ARect.left && AX < ARect.right && AY >= ARect.top && AY < ARect.bottom;
}
//---------------------------------------------------------------------------
inline bool __fastcall TGameUnit::Contains(const TRect& ARect1, const TRect& ARect2) const
{
	return
		Contains(ARect1, ARect2.left, ARect2.top) ||
		Contains(ARect1, ARect2.right, ARect2.top) ||
		Contains(ARect1, ARect2.left, ARect2.bottom) ||
		Contains(ARect1, ARect2.right, ARect2.bottom);
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameUnit::TGameUnit(GAMEUNIT AType, int ALayer) : FType(AType),
	FLayer(ALayer), FAlive(true), FBounds(0, 0, 0, 0), FDX(0), FDY(0)
{
	GameSystem->UnitList->Add(this);
}
//---------------------------------------------------------------------------
__fastcall TGameUnit::~TGameUnit()
{
	GameSystem->UnitList->Remove(this);
}
//---------------------------------------------------------------------------
bool __fastcall TGameUnit::IsHit(int AX, int AY)
{
	return Contains(Bounds, AX, AY);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TTableUnit : public TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TTableUnit::TTableUnit() : TGameUnit(GAMEUNIT_TABLE, 0)
{
	Bounds = SCRR; 
}
//---------------------------------------------------------------------------
__fastcall TTableUnit::~TTableUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTableUnit::Move()
{
	// draw
	TCanvas* canvas = GameScreen->SrcCanvas;
	canvas->Brush->Color = static_cast<TColor>(RGB(0, 192, 0));
	canvas->FillRect(SCRR);
	int blockw = SCRR.Width() / 4;
	int blockh = SCRR.Height() / 4;
	int left = -(blockw / 2);
	int top = -(blockh / 2);
	for(int i = 0; i < 5; i++) {
		for(int j = 0; j < 5; j++) {
			canvas->Brush->Color = static_cast<TColor>(RGB(0, 224, 0));
			canvas->FillRect(TRect(
				left + blockw * i,
				top + blockh * j,
				left + blockw * i + blockw / 4 * 3,
				top + blockh * j + blockh / 4 * 3));
			canvas->Brush->Color = static_cast<TColor>(RGB(0, 160, 0));
			canvas->FillRect(TRect(
				left + blockw * i + blockw / 4 * 3,
				top + blockh * j + blockh / 4 * 3,
				left + blockw * i + blockw,
				top + blockh * j + blockh));
		}
	}
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TForkUnit : public TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TForkUnit::TForkUnit() : TGameUnit(GAMEUNIT_FORK, 3)
{
	switch(random(4)) {
		case 0: FPattern = PATTERN_W; break;
		case 1: FPattern = PATTERN_E; break;
		case 2: FPattern = PATTERN_N; break;
		default: FPattern = PATTERN_S; break;
	}

	if(FPattern == PATTERN_W || FPattern == PATTERN_E) {
		FBounds.left = FPattern == PATTERN_W ? (SCRR.Width() + random(HORZW)) : -(HORZW + random(HORZW));
		FBounds.right = FBounds.left + HORZW;
		FBounds.top = random(SCRR.Height() + HORZH * 5 + 1) - HORZH * 2;
		FBounds.bottom = FBounds.top + HORZH;
		FDX = (12 + random(12)) * (FPattern == PATTERN_W ? -1 : 1);
		FDY = 0;
	}
	else {
		FBounds.left = random(SCRR.Height() + VERTW * 5 + 1) - VERTW * 2;
		FBounds.right = FBounds.left + VERTW;
		FBounds.top = FPattern == PATTERN_N ? (SCRR.Height() + random(VERTH)) : -(VERTH + random(VERTH));
		FBounds.bottom = FBounds.top + VERTH;
		FDX = 0;
		FDY = (12 + random(12)) * (FPattern == PATTERN_N ? -1 : 1);
	}
}
//---------------------------------------------------------------------------
__fastcall TForkUnit::~TForkUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TForkUnit::Move()
{
	// move
	FBounds.left += DX;
	FBounds.right += DX;
	FBounds.top += DY;
	FBounds.bottom += DY;

	// remove if out of screen
	if(!Contains(SCRR, FBounds)) {
		FAlive = false;
		new TForkUnit();
		return;
	}

	// draw
	int x = (FBounds.left + FBounds.right) / 2;
	int y = (FBounds.top + FBounds.bottom) / 2;
	switch(FPattern) {
		case PATTERN_W:
			x -= UNITW * 3 / 2;
			y -= UNITH / 2;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 0); x += UNITW;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 8); x += UNITW;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 4);
			break;

		case PATTERN_E:
			x -= UNITW * 3 / 2;
			y -= UNITH / 2;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 5); x += UNITW;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 8); x += UNITW;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 1);
			break;

		case PATTERN_N:
			x -= UNITW / 2;
			y -= UNITH * 3 / 2;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 2); y += UNITH;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 9); y += UNITH;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 6);
			break;

		case PATTERN_S:
			x -= UNITW / 2;
			y -= UNITH * 3 / 2;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 7); y += UNITH;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 9); y += UNITH;
			GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, x, y, 3);
			break;
	}
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TDishUnit : public TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TDishUnit::TDishUnit() : TGameUnit(GAMEUNIT_DISH, 1), FPattern(random(4))
{
	Bounds.left = WIDTH + random(SCRR.Width() - WIDTH * 3 + 1);
	Bounds.right = Bounds.left + WIDTH;
	Bounds.top = HEIGHT + random(SCRR.Height() - HEIGHT * 3 + 1);
	Bounds.bottom = Bounds.top + HEIGHT;
}
//---------------------------------------------------------------------------
__fastcall TDishUnit::~TDishUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDishUnit::Move()
{
	// draw
	GameScreen->UnitImages->Draw(GameScreen->SrcCanvas,
		(FBounds.left + FBounds.right) / 2 - UNITW / 2,
		(FBounds.top + FBounds.bottom) / 2 - UNITH / 2, 14 + FPattern);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TMarkUnit : public TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMarkUnit::TMarkUnit(int AX, int AY) : TGameUnit(GAMEUNIT_MARK, 2),
	FX(AX), FY(AY), FPattern(random(4)), FAliveCount(12)
{
	// do nothing
}
//---------------------------------------------------------------------------
__fastcall TMarkUnit::~TMarkUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TMarkUnit::Move()
{
	if(--FAliveCount < 0) {
		FAlive = false;
		return;
	}
	FPattern = (FPattern + 1) % 4;

	// draw
	GameScreen->UnitImages->Draw(GameScreen->SrcCanvas, FX - UNITW / 2, FY - UNITH / 2, 10 + FPattern);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TMessageUnit : public TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TMessageUnit::DrawTitle()
{
	AnsiString text = Application->Title;

	TCanvas* canvas = GameScreen->SrcCanvas;
	canvas->Brush->Color = static_cast<TColor>(RGB(0, 192, 0));
	canvas->Font->Color = static_cast<TColor>(RGB(255, 255, 255));
	canvas->TextRect(SCRR, (SCRR.Width() - canvas->TextWidth(text)) / 2,
		(SCRR.Height() - canvas->TextHeight(text)) / 2, text);
}
//---------------------------------------------------------------------------
void __fastcall TMessageUnit::DrawReady()
{
	AnsiString text = "Ready.";

	TCanvas* canvas = GameScreen->SrcCanvas;
	canvas->Brush->Color = static_cast<TColor>(RGB(0, 192, 0));
	canvas->Font->Color = static_cast<TColor>(RGB(255, 255, 255));
	canvas->TextRect(SCRR, (SCRR.Width() - canvas->TextWidth(text)) / 2,
		(SCRR.Height() - canvas->TextHeight(text)) / 2, text);
}
//---------------------------------------------------------------------------
void __fastcall TMessageUnit::DrawGameOver()
{
	bool highScore = GameSystem->Score == GameSystem->HighScore;

	AnsiString text = AnsiString(" ") + (highScore ? "High-score!!" : "Game Over!") +
		"  Score: " + GameSystem->Score + " ";

	TCanvas* canvas = GameScreen->SrcCanvas;
	canvas->Brush->Color = static_cast<TColor>(RGB(0, 224, 0));
	canvas->Font->Color = static_cast<TColor>(RGB(255, 255, 255));

	TRect rect((SCRR.Width() - canvas->TextWidth(text)) / 2, (SCRR.Height() -
		canvas->TextHeight(text)) / 2, 0, 0);
	rect.right = rect.left + canvas->TextWidth(text);
	rect.bottom = rect.top + canvas->TextHeight(text);

	canvas->TextOut(rect.left, rect.top, text);

	// draw icons
	GameScreen->UnitImages->Draw(canvas, rect.left - UNITW, (SCRR.Height() - UNITH) / 2,
		highScore ? 18 : 19);
	GameScreen->UnitImages->Draw(canvas, rect.right, (SCRR.Height() - UNITH) / 2,
		highScore ? 20 : 21);
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMessageUnit::TMessageUnit() : TGameUnit(GAMEUNIT_MESSAGE, 4)
{
	// do nothing
}
//---------------------------------------------------------------------------
__fastcall TMessageUnit::~TMessageUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TMessageUnit::Move()
{
	switch(GameSystem->Mode) {
		case GAMEMODE_TITLE: DrawTitle(); break;
		case GAMEMODE_READY: DrawReady(); break;
		case GAMEMODE_GAMEOVER: DrawGameOver(); break;
	}

	Alive = false;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGameUnitList
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
int __fastcall TGameUnitList::GetCount()
{
	return FList->Count;
}
//---------------------------------------------------------------------------
TGameUnit* __fastcall TGameUnitList::GetUnit(int AIndex)
{
	return static_cast<TGameUnit*>(FList->Items[AIndex]);
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameUnitList::TGameUnitList() : FList(NULL)
{
	FList = new TList();
}
//---------------------------------------------------------------------------
__fastcall TGameUnitList::~TGameUnitList()
{
	Clear();

	delete FList; FList = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TGameUnitList::Add(TGameUnit* AUnit)
{
	int index = -1;
	while(++index < Count)
		if(Units[index]->Layer > AUnit->Layer) break;

	FList->Insert(index, AUnit);
}
//---------------------------------------------------------------------------
void __fastcall TGameUnitList::Remove(TGameUnit* AUnit)
{
	FList->Remove(AUnit);
}
//---------------------------------------------------------------------------
void __fastcall TGameUnitList::Clear()
{
	while(Count > 0)
		delete Units[Count - 1];
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
#pragma package(smart_init)
