//---------------------------------------------------------------------------
#ifndef TGameUnitH
#define TGameUnitH
#include <Classes.hpp>
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
enum GAMEUNIT { GAMEUNIT_TABLE, GAMEUNIT_FORK, GAMEUNIT_DISH, GAMEUNIT_MARK,
	GAMEUNIT_MESSAGE };
//---------------------------------------------------------------------------
class TGameUnit
{
private:
	GAMEUNIT FType;
	int FLayer;

protected:
	bool FAlive;
	TRect FBounds;
	int FDX, FDY;

	inline bool __fastcall Contains(const TRect& ARect, int AX, int AY) const;
	inline bool __fastcall Contains(const TRect& ARect1, const TRect& ARect2) const;

public:
	__fastcall TGameUnit(GAMEUNIT AType, int ALayer);
	__fastcall virtual ~TGameUnit();

	bool __fastcall virtual IsHit(int AX, int AY);
	void __fastcall virtual Move() = 0;

	__property GAMEUNIT Type = {read=FType};
	__property int Layer = {read=FLayer};
	__property bool Alive = {read=FAlive, write=FAlive};
	__property TRect Bounds = {read=FBounds, write=FBounds};
	__property int DX = {read=FDX, write=FDX};
	__property int DY = {read=FDY, write=FDY};
};
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
class TTableUnit : public TGameUnit
{
private:

public:
	__fastcall TTableUnit();
	__fastcall ~TTableUnit();

	void __fastcall Move();
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
class TForkUnit : public TGameUnit
{
private:
	enum PATTERN { PATTERN_W, PATTERN_E, PATTERN_N, PATTERN_S } FPattern;
	static const int HORZW = 92;
	static const int HORZH = 20;
	static const int VERTW = HORZH;
	static const int VERTH = HORZW;

public:
	__fastcall TForkUnit();
	__fastcall ~TForkUnit();

	void __fastcall Move();
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
class TDishUnit : public TGameUnit
{
private:
	static const int WIDTH = 30;
	static const int HEIGHT = 30;
	int FPattern;

public:
	__fastcall TDishUnit();
	__fastcall ~TDishUnit();

	void __fastcall Move();
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
class TMarkUnit : public TGameUnit
{
private:
	int FX, FY;
	int FPattern;
	int FAliveCount;

public:
	__fastcall TMarkUnit(int AX, int AY);
	__fastcall ~TMarkUnit();

	void __fastcall Move();
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
class TMessageUnit : public TGameUnit
{
private:
	void __fastcall DrawTitle();
	void __fastcall DrawReady();
	void __fastcall DrawGameOver();

public:
	__fastcall TMessageUnit();
	__fastcall ~TMessageUnit();

	void __fastcall Move();
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
class TGameUnitList
{
private:
	TList* FList;

	int __fastcall GetCount();
	TGameUnit* __fastcall GetUnit(int AIndex);

public:
	__fastcall TGameUnitList();
	__fastcall virtual ~TGameUnitList();

	void __fastcall Add(TGameUnit* AUnit);
	void __fastcall Remove(TGameUnit* AUnit);
	void __fastcall Clear();

	__property int Count = {read=GetCount};
	__property TGameUnit* Units[int AIndex] = {read=GetUnit};
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
#endif
